
# q-expansion of newform 9675.2.a.ch, downloaded from the LMFDB on 13 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9675
weight = 2
poly_data = [-4, 5, 13, -7, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-4, 0, 1, 0, 0], 1], [[0, -6, 0, 1, 0], 1], [[1, 0, -6, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [-1, 0, 0, 1, -1], [1, 0, 0, -1, 0], [0, -1, 1, 0, 1], [-4, 1, -1, 0, 1], [0, -2, 2, 0, -2], [0, -1, -1, 0, -1], [-2, 0, -2, -2, 2], [1, 1, -1, 1, 2], [-1, -1, -1, -1, 1], [0, -2, -1, 1, -1], [1, 0, 0, 0, 0], [0, -2, -2, 0, 0], [-4, -1, 1, 0, -1], [0, 1, 0, 1, 0], [2, 2, -2, 0, -2], [-6, 3, -1, -2, 1], [-2, 2, -2, -2, 4], [0, -1, 2, -1, 0], [9, -1, 1, 1, -1], [-2, 3, 1, 0, 1], [-4, 2, 2, 2, -2], [-6, -1, 3, 0, -1], [-1, -1, 1, 1, -2], [2, -3, 3, 0, -3], [-2, 2, -2, 2, 2], [1, 5, -1, -5, 2], [7, -2, 2, -1, 1], [-6, -3, 1, 2, -3], [0, 0, -2, -2, 0], [8, -5, 0, 1, -2], [0, -2, 1, -3, -1], [-2, 2, -4, 2, 2], [-4, 0, 2, 2, 2], [8, -3, 0, 3, -2], [3, -3, 3, -1, -3], [-12, 3, -1, 0, 3], [2, 0, -2, -2, 4], [6, -2, 4, 0, 0], [4, 5, 0, 3, -2], [8, -2, 0, -2, 4], [-14, 3, -1, 0, -1], [-4, -4, 0, 6, -2], [-12, 4, -2, 2, 2], [10, -2, 6, 2, 0], [-4, -2, -2, -4, -2], [-1, -6, -2, -7, 1], [-7, 6, -4, 3, 4], [-4, -3, -4, -5, -2], [-5, 2, -2, -7, -3], [4, -4, 0, 2, -4], [4, 3, 1, 2, -1], [-1, -3, 1, 3, 3], [-1, 3, 5, 3, -1], [-5, 8, -2, 5, 2], [0, 4, -3, 3, 3], [1, 0, 4, -3, 1], [5, -2, 0, -5, 4], [-4, -3, -5, -6, 7], [0, -2, 0, -4, 2], [-14, -1, -3, 4, 1], [4, 2, -1, 3, 1], [-9, 10, -2, 3, 5], [-2, -1, 1, -2, -1], [-4, 2, -4, -2, 0], [-4, -5, -1, -10, -1], [3, -1, 5, -1, -1], [2, -4, 2, -2, -6], [-4, 5, 3, 0, 1], [-8, 9, 3, 6, 1], [-10, 0, 2, 0, -6], [9, 6, 6, 9, -5], [11, 0, 0, 5, 0], [-12, 1, 4, -3, -4], [-14, -8, 2, -2, 0], [6, -8, 2, 2, -8], [-11, 5, 3, 3, -2], [2, -2, -2, 0, 0], [-20, 4, -2, -2, -2], [4, -8, 8, 2, -6], [16, -4, -1, 1, 1], [-19, 3, -1, 1, 1], [1, -12, 4, 3, 0], [4, 10, 4, -2, 0], [10, -6, 0, -2, 2], [-8, -1, 2, 7, -8], [-7, -1, 3, 1, -3], [-8, 11, 2, 5, 2], [-4, 10, -2, -4, 2], [13, -3, -1, 1, 2], [-2, -4, -2, 0, 6], [4, 6, 0, -2, 4], [2, 10, 4, 8, -4], [7, -4, 4, 1, -1], [11, -7, 7, 1, -8], [-4, 2, 8, 4, -4], [9, 1, -1, 1, 9], [11, 4, 2, -7, 2], [-2, 7, -3, -8, -1], [-14, 7, 3, 0, -5], [-4, 1, -1, 2, 5], [-1, 15, 1, -3, 2], [2, -2, -2, 2, -6], [-6, 12, -4, 4, 8], [-13, -4, -8, -11, 7], [-1, 1, 1, 7, 3], [7, -5, 5, 7, 2], [-8, 0, 4, 6, 0], [-12, -1, -4, -5, 8], [-8, -4, -6, -12, 2], [10, 1, -3, 4, 5], [-3, -6, -2, -5, -1], [2, -8, -4, -6, 0], [-8, -14, 4, -4, -2], [-2, 6, -10, -2, 6], [11, 2, 10, -3, 1], [-23, 4, -4, -11, -3], [16, 0, 5, 3, 7], [-1, -6, 0, 5, 0], [-7, 5, 1, 5, -9], [-17, -3, 1, 7, -1], [-2, 11, 3, -2, 1], [-12, 10, 0, -6, 2], [-7, -1, -5, -7, 5], [-16, 6, -1, -7, -5], [-13, -11, 3, 3, -3], [-9, 9, -5, -5, -3], [18, -2, 6, 8, -2], [-10, 0, 2, 4, 8], [12, 7, 0, -1, 4], [8, -8, 6, 10, 0], [3, -2, -6, 3, 1], [24, -2, -4, 0, 4], [7, 4, 4, 3, -1], [-21, 4, -8, -1, 3], [-6, 12, 6, 0, 2], [-12, -14, 2, 2, -2], [-8, 9, -16, 3, 6], [-18, 16, 6, 8, -2], [20, 10, -5, 1, -1], [0, 17, -9, 10, 5], [20, -2, 6, 0, 0], [-2, 14, -4, 2, 2], [4, -4, 0, -8, 10], [8, 3, -1, 6, 1], [-18, 2, -8, -2, 0], [-8, -10, 0, 2, -4], [11, -12, -8, -7, -1], [-4, -3, 5, -2, -5], [3, -7, -5, 1, 4], [24, -7, -1, -10, -3], [-24, 2, 2, 8, -2], [-18, 1, 3, -4, 3], [-10, 4, 4, -6, -2], [0, 8, 1, -5, -1], [4, 10, 4, -8, -2], [11, -4, 0, -1, -9], [6, 5, 7, 10, -3], [8, -7, 5, 0, -5], [-17, 14, 2, -1, -3], [-4, 7, 3, 8, 3], [-21, -9, -3, 3, -2], [-20, 10, -10, -14, 10], [25, 5, -9, 1, 9], [-18, -6, 0, 4, -6], [-9, 1, 1, 7, 3], [4, 10, -6, 6, -2], [-10, 2, -2, -4, -4], [-30, 0, -4, -2, -4], [10, -6, 10, 12, -8], [-8, -2, -12, 2, 0], [-2, 1, 7, 2, 1], [-18, -4, -14, -12, 4], [5, -9, 3, 1, -11], [14, -8, 6, 0, -10], [-12, 10, -2, 10, -6], [6, -6, -2, -2, -14], [18, -6, 2, 4, 2], [-4, 7, 2, -3, 10], [-3, -18, 2, -5, -1], [18, -6, -2, -12, -2], [-19, 8, -12, -7, 5], [5, -5, 1, 9, -9], [-26, 6, -2, -4, -2], [0, 11, -4, 5, 6], [-3, -8, 8, -9, 1], [0, -13, 0, -15, 6], [-2, -18, 0, 8, -2], [6, 5, 3, -6, 9], [-8, 9, -6, -1, -6], [9, 18, -2, -1, 3], [-24, 4, -2, -8, 8], [-28, 17, -2, -1, 6], [12, -11, 10, -3, 0], [5, 2, 0, -5, 8], [4, 7, -5, 2, 7], [0, 0, 10, 8, -14], [8, 13, -4, 5, 4], [-19, -1, 1, -5, 8], [-38, 4, 0, 2, 4], [-17, 6, 2, -9, 5], [-4, 4, 2, 8, -10], [2, 8, 12, 6, 4], [-15, 9, -7, -7, 7], [-2, -14, -4, -16, 6], [0, 5, 3, 6, 3], [10, 6, 4, 6, -8], [-27, -2, -2, -13, 2], [18, -8, 4, 0, -12], [-26, 7, 5, 6, -1], [-2, 9, 9, 2, -9], [28, -5, 9, 2, -11], [10, 10, -12, -4, 18], [8, 2, -1, 1, 3], [7, 13, -9, -5, 1], [-28, 0, -6, -8, 0], [24, -14, -6, 0, 2], [10, 1, -1, 6, 1], [-10, 4, 8, -4, -8], [16, -15, -4, -7, -12], [-12, 4, 2, 4, -16], [0, -7, -1, -6, 7], [-8, 11, 0, 3, -8], [-9, 4, 6, 1, -2], [-16, 11, 3, 18, -3], [26, 2, 6, 10, -10], [-8, -3, -6, -1, -2], [-8, 0, 12, 8, -18], [-1, -17, -9, -1, -3], [-8, -2, 7, -1, 5], [-17, -11, 11, -5, -12], [-16, 4, -8, 10, 4], [-4, -9, 1, -2, -15], [-20, -14, 6, 0, -8], [-10, -4, -6, 6, -14], [4, -9, -5, -2, 9], [24, -18, -6, -8, -8], [-28, -9, -3, 14, -9], [23, 6, 6, -11, 1], [-18, -12, -8, -6, 6], [18, -10, -6, 2, -10], [-15, 3, -1, 13, -19], [-18, -14, 10, -4, 6], [-21, 1, 3, -1, -3], [17, -17, -7, -7, -5], [27, 5, 11, 11, -8], [-11, -1, -7, -3, 4], [4, -13, 9, 6, -15], [-26, -8, 4, 0, -12], [-3, -16, 2, 11, -2], [-20, 11, -5, -4, 5], [34, -12, -2, -2, -12], [0, -21, 4, 9, -6], [16, -12, 10, 6, -16], [-6, 12, 2, 14, 10], [24, -5, 15, 8, -11], [-29, 12, -16, -9, 11], [-20, -3, -11, -14, 3], [16, 2, -6, 0, 18], [-12, -2, -2, -8, 18], [-1, 0, -8, 7, 3], [16, 16, -2, 4, 2], [3, -2, 2, 3, 1], [-24, 22, 2, 0, 6], [0, -18, -8, -16, 6], [-24, 8, 5, -3, -11], [-7, -23, 3, 5, 1], [36, 17, 3, 8, -3], [-20, 12, 5, 5, 5], [15, 4, 4, 5, -5], [-5, 1, 1, -17, 7], [-4, 20, -8, 8, 10], [6, 2, 12, -10, 0], [-12, -3, 4, 1, -4], [-20, -3, -7, 4, -9], [-10, 14, -6, -4, 14], [5, -13, -7, 7, 8], [-18, 8, 4, 2, 14], [-26, -13, 5, 8, -9], [-12, 9, -4, 13, 0], [4, 9, -13, 2, 5], [-40, 10, -16, -16, 6], [27, 6, -2, -7, 13], [-40, 22, -2, 0, 14], [2, 3, 11, 12, -3], [32, -3, 4, 11, -2], [14, -18, -8, -8, 2], [-1, -8, -6, -15, -10], [24, -10, 9, 11, -7], [-26, 11, -3, -12, 1], [10, -12, -14, 2, 8], [14, -6, 18, -2, 4], [26, -3, 17, 12, -17], [-14, 17, -17, 2, 9], [-52, 3, -15, -10, 7], [12, -27, 7, 8, -13], [-16, -20, 4, -4, -4], [4, 12, -6, 18, 4], [0, 17, 1, -10, 3], [4, -9, 18, -5, 0], [4, -9, 5, -2, 3], [26, 15, -15, 0, 17], [20, 20, -4, -8, 6], [-36, 21, 0, 3, 18], [8, -7, 11, 10, -7], [10, -12, 20, -4, -2], [6, 8, -12, -2, 16], [16, -23, 6, 1, -4], [-44, 9, -10, -5, 2], [18, -10, 6, 0, 4], [7, -12, -6, -11, -2], [24, -9, 2, -3, -2], [-10, -25, 3, -10, 7], [1, -29, -7, -21, 8], [-12, -5, 4, 3, -20], [-5, 18, -14, -11, 5], [-40, 3, -5, -2, 5], [-18, 10, -4, -10, -12], [20, -21, 9, 2, -9], [-25, -1, -13, -1, 5], [-36, 18, -2, 2, 14], [-6, 10, 4, 12, -22], [-26, 14, -14, -16, 10], [12, 18, 1, 11, -11], [18, 6, 6, 2, 0], [12, 11, -5, 0, -1], [16, 2, -12, -2, 24], [-5, 13, -15, -5, 11], [-22, -4, 6, -6, 8], [-14, 17, -11, 8, 13], [18, -20, 2, 8, -6], [29, 7, -15, 1, -5], [-21, 6, 10, -3, 1], [-6, 3, -5, -22, -9], [10, 8, 0, 20, -6], [20, -13, 4, 1, 2], [8, 14, 0, 16, 12], [0, -12, -8, 4, -4], [-20, 1, -17, 8, -15], [15, 7, -1, -9, 21], [2, -2, 4, 18, 6], [-16, -4, 8, -4, 4], [-40, 11, 5, -4, -15], [16, 4, 7, -3, 5], [-4, -8, -6, 8, -2], [0, 2, 10, 12, 12], [34, -12, 16, 18, -22], [-32, 29, 1, 10, 13], [35, -3, -9, -5, 21], [-42, 6, -6, -4, -2], [2, -16, 0, 16, -4], [13, -19, 11, -7, 1], [31, -8, 16, -9, 3], [-18, 8, -8, -4, -8], [18, 22, -10, 6, -2], [-10, -6, -8, -8, 14], [-6, 11, -3, 14, -7], [28, -9, 9, -6, -11], [8, 9, -8, 15, 18], [30, 8, 0, 0, 8], [1, 8, 8, 7, 1], [12, -32, -8, 8, 2], [-10, 19, 7, -14, -1], [-3, 6, -10, 17, 7], [25, -11, 1, 5, -13], [-20, 12, -2, 18, -12], [50, 8, -2, 2, -4], [8, -14, -10, 0, -2], [6, -20, 10, 16, -8], [-8, 22, -4, 18, 0], [-4, 12, -18, -4, 10], [-37, 6, -18, 9, 6], [-3, 10, -18, 9, 3], [-2, -4, -8, -6, -8], [5, -2, 2, 7, -21], [-36, -3, -1, 0, -1], [-10, -6, -8, 10, -10], [31, -24, 20, 3, -21], [-17, -4, 2, -15, 10], [24, -3, 5, -14, 1], [-10, -15, 13, -2, -1], [-17, -7, 1, -1, 3], [45, -9, -3, 1, -8], [-20, 13, 9, -8, -11], [38, 2, 4, 24, 8], [-24, 7, 15, 2, -17], [-44, -8, 4, -4, -8], [24, -19, 25, 4, -15], [23, -12, -10, 13, 10], [-44, 7, -4, 1, 6], [-16, 8, 12, 0, 12], [-14, -9, -1, 2, -13], [-22, 12, -4, 30, -2], [-10, 4, 18, 2, -4], [28, 1, 15, 22, -11], [-32, 0, -6, 4, 4], [-38, 12, -10, -16, 16], [26, -8, 2, -2, 6], [-8, -4, 9, 5, -11], [-4, -12, 0, -6, -10], [42, 11, 1, 8, 9], [30, -4, 0, 8, 4], [-32, -12, -4, 2, 6], [-12, 2, -2, -16, 8], [-14, 0, 0, -16, -4], [47, 0, 8, 15, -13], [19, -22, -6, -15, 1], [41, -4, -8, 3, 8], [16, -10, 10, 18, 2], [-12, -13, 16, 9, -22], [29, -3, -5, 5, 1], [-12, -12, -5, -19, 9], [-7, -9, 11, 5, 9], [-24, -4, 15, -5, -9], [47, -9, 9, 15, -10], [30, -10, -4, 4, 2], [50, -6, 20, 14, -12], [15, 14, -14, -3, -7], [-2, 14, -4, 22, 10], [10, -16, 8, 8, -16], [-24, -7, 16, 5, 4], [-5, 0, -4, 17, -4], [-7, -25, 11, 5, -11], [-17, -1, 7, -5, -3], [19, 4, 4, -7, 11], [0, -2, -4, -24, 8], [-11, 0, 0, 19, -20], [18, 4, 12, -10, 0]]
